#This function perform the Bayesian Ising Approximation for a dependent variable Y and 
#an independent feature matrix X with rows different observatons and columns different 
#features.
#It returns a list with elements:
#rank (ordering of features at threshold)
#pp (posterior probability of list at threshold)
#pppath (the path of pp as threshold is varied)
#plot (a plot of the feature selection path using gg plot)
#epsilonpath ( \lambda^* over \lamba which varies from one to three)
#threshold (the threshold of the penalty parameter where the BIA starts to break down) 

PerformBIA<-function(y,X){

  
require("ggplot2")
require("reshape2")
require("scales")
# Find number of data points
n <-dim(X)[[1]]
# Find number of features
p<-dim(X)[[2]]

#Standerdize the data

scaled.y <-scale(y)
scaled.X<-scale(X)

# Calculate correlation matrix

Xcorrelations <-cor(scaled.X)
YXcorrelations<-cor(scaled.y, scaled.X)


#Calculate average correlation parameter
r<-sqrt((sum(Xcorrelations*Xcorrelations)-p)/(p*(p-1)))
#Calculate threshold lambda and epsilon
lambdath <-n*(1+p*r)
epsilonth <-1/lambdath



#Calculate magnetizations
depsilon=0.05*epsilonth

#Set up magnetization vector
m=matrix(rep(0, p*61), nrow=p, ncol=61)


#intialize epsilon
for (s in 1:60){
  epsilon=s*depsilon 
  #Calculate J using formula
  J<-epsilon*n*(  (Xcorrelations*Xcorrelations)/n - (t(YXcorrelations)%*%(YXcorrelations))*Xcorrelations
                  +0.5*t(YXcorrelations^2)%*%(YXcorrelations^2))
  #Create J with zero diagonal
  J0<-J
  diag(J0)<-0
  #Calculate h using formula
  
  h<-YXcorrelations^2 -1/n+rowSums(J)
  
  m[,s+1]<-tanh((epsilon)*n^2/4*(h+0.5*t(J0 %*% m[,s])))
}

epsilonpath<-depsilon*(0:60)/epsilonth



#Plot Resulting Graph

  library (scales)
  library(ggplot2)
  library(reshape2)
  
 
  df <-data.frame(epsilonpath,t(m))
  #melt the data to a long format
  df2 <- melt(data = df, id.vars = "epsilonpath")
  # plot, using the aesthetics argument 'colour'
  lp<-ggplot(data = df2, aes(x = epsilonpath, y = value, colour = variable)) + geom_line()
  lp<-lp+xlab(expression(frac(lambda^"*", lambda)))+ylab("Posterior Probabilities")
  lp
result<-list(ranks=order(-m[,21]), pp=(1+m[order(-m[,21]), 21])/2, epsilonpath=epsilonpath, pppath=(1+m)/2, plot=lp, threshold=lambdath)

return(result)
}

